
#include "bits/bitcombminchange.h"

#include "fxttypes.h"
#include "bits/printbin.h"
#include "jjassert.h"
#include "demo/nextarg.h"

#include "fxtio.h"

//% Generating all combinations of bits (as binary words) in minimal-change order.

//#define TIMING // uncomment to disable printing

int
main(int argc, char **argv)
{
    ulong n = 7, k = 4;
    NXARG(n, "Minchange combinations (n over k):  n  (n>0)");
    NXARG(k, "  (0<k<=n)");

    jjassert( n>0 );
    jjassert( k>0 );
    jjassert( n>=k );

    const ulong pd = n;
    ulong last = igc_last_comb(k, n);
    ulong c, nc = first_sequency(k);
    ulong g, gg = 0;
    ulong ct = 0;
    do
    {
        ++ct;
        c = nc;
        nc = igc_next_minchange_comb(c);  // 1-arg version
//        nc = igc_next_minchange_comb(c, k);  // 2-arg version

#ifndef TIMING
        print_bin_nn("    ", c, pd);
        print_bin_nn("    ", nc-c, pd);
        g = gray_code(c);
        print_bin_nn("    ", g, pd);
        if ( gg )  print_bin_diff_nn("    ", gg, g, pd, ".1+-");
        gg = g;
        cout << endl;
#endif // TIMING
    }
    while ( c!=last );

    cout << " ct = " << ct << endl;

    return 0;
}
// -------------------------

/*
Timing:
 % time ./bin 32 12
-------- 1-arg igc_next_minchange_comb() jj:
./bin 32 12  2.47s user 0.02s system 99% cpu 2.498 total
 ct = 225792840
 ==> 225792840/2.47  == 91,414,105 comb/sec
//
 % time ./bin 32 20
./bin 32 20  2.85s user 0.01s system 99% cpu 2.863 total
 ct = 225792840
 ==> 225792840/2.85 == 79,225,557 comb/sec

check symmetry:
%  time ./bin 63 6   ct = 67945521
./bin 63 6  1.84s user 0.00s system 99% cpu 1.842 total
%  time ./bin 63 57  ct = 67945521
./bin 63 57  2.36s user 0.02s system 99% cpu 2.385 total

-------- 1-arg igc_next_minchange_comb() Doug Moore:
 % time ./bin 32 12
./bin 32 12  3.48s user 0.02s system 99% cpu 3.506 total
 ==> 225792840/3.48 ==  64,883,000 comb/sec

 % time ./bin 32 20
./bin 32 20  2.84s user 0.02s system 99% cpu 2.863 total
 ==> 225792840/2.84 == 79,504,521  comb/sec

------ 2-arg igc_next_minchange_comb():
%  time ./bin 32 12
./bin 32 12  3.65s user 0.02s system 99% cpu 3.674 total
 ==> 225792840/3.65 == 61,861,052 comb/sec

 % time ./bin 32 20
./bin 32 20  3.43s user 0.02s system 100% cpu 3.445 total
 ==> 225792840/3.43 ==  65,828,816 comb/sec
*/


/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/bits"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/bits/bitcombminchange-demo.cc"
/// End:

