
// demo-is-self-contained

#include "fxtio.h"
#include "demo/nextarg.h"

//% Generate binary Lyndon words ordered so that only few changes
//% between successive elements occur (note: in general not a Gray code).
//% Recursive CAT algorithm.

ulong *f; // data in f[1..N],  f[0] = 0
ulong N; // word length

ulong ct;  // count objects

//#define TIMING // uncomment to disable printing

void
visit(ulong p)
{
    if ( N!=p )  return;  // only Lyndon words
//    if ( (N%p)!=0 )  return;  // only necklaces

    ++ct;
#ifndef TIMING
    cout << setw(3) << ct << ":  ";
    for (ulong j=1; j<=N; ++j)
    {
//        cout << ' ';
        if ( f[j]==0 )  cout << '.';
        else            cout << f[j];
    }
//    if ( N%p==0 )  cout << "  N";
//    if ( N==p )  cout << "  L";
    cout << endl;
#endif  // TIMING
}
// -------------------------

void xgen(ulong n, ulong p, int x=+1)
{
    if ( n > N )  visit(p);
    else
    {
        if ( -1==x )
        {
            if ( 0==f[n-p] )  { f[n] = 1;  xgen(n+1, n, -x); }
            f[n] = f[n-p];  xgen(n+1, p, +x);
        }
        else
        {
            f[n] = f[n-p];  xgen(n+1, p, +x);
            if ( 0==f[n-p] )  { f[n] = 1;  xgen(n+1, n, -x); }
        }
    }
}
// -------------------------

int
main(int argc, char **argv)
{
    N = 8;
    NXARG(N, "Length of words (n>=1)");
    
    f = new ulong[N+1];
    for (ulong j=0; j<=N; ++j)  f[j] = 0;
    ct = 0;
    xgen(1, 1);
    cout << "  ct =" << ct << endl;

    return 0;
}
// -------------------------

/*
Timing: (with Lyndon words)
 time ./bin 32 2
arg 1: 32 == N  [Length of words (n>=1)]  default=8
  ct =134215680
./bin 32 2  3.94s user 0.02s system 99% cpu 3.964 total
 ==> 134215680/3.94 == 34,064,893 words per second
*/

/// Emacs:
/// Local Variables:
/// MyRelDir: "demo/comb"
/// makefile-dir: "../../"
/// make-target: "1demo DSRC=demo/comb/necklace-gray-demo.cc"
/// End:

