
#include "bmat/bitmat.h"
#include "bpol/bitpol-irred.h"

#include "fxtalloca.h"

#include "fxttypes.h"  // ulong
//#include "fxtiomanip.h"


// defines to make the code less cryptic:
#define mGET(M, i, j)  ((M[i]>>j) & 1UL)
#define mSET(M, i, j, s) { ulong bj=1UL<<j; if (s) {M[i]|=bj;}  else { M[i]&=~bj; } }

ulong
bitpol_normal_q(ulong c, ulong n,
                ulong iq/*=0*/,
                ulong *M/*=0*/)
// Determine whether polynomial c (of degree n) is normal.
// Set iq to one for polynomials known to be irreducible.
// Return 1 for normal polynomial, else zero.
// If M is given then the multiplication matrix is computed.
{
    ulong h = 1UL << (n-1);
    if ( 0==(c & h) )  return 0; // polynomial trace must be one
    if ( 0==iq )
    {
        iq = bitpol_irreducible_q(c, h);
        if ( 0==iq )  return 0;  // polynomial reducible ==> not normal
    }

    ALLOCA(ulong, A, n);
    ulong v = 2UL; // 'x'
    for (ulong k=0; k<n; ++k)
    {
        A[k] = v;
        v = bitpolmod_square(v, c, h);
    }

    ALLOCA(ulong, Ai, n);
    ulong q = bitmat_inverse(A, n, Ai);  // try to invert
    if ( 0==q )  return 0;  // polynomial is not normal

    if ( 0!=M )  // compute multiplication matrix
    {
        ALLOCA(ulong, C, n);
        bitmat_companion(c, n, C);
        bitmat_transpose(C, n, C);

//        bitmat_print("A=", A, n);
//        bitmat_print("A^-1=", Ai, n);
//        bitmat_print("C^T=", C, n);

        bitmat_mult(A, C, n, A);  // A*C^T
        bitmat_mult(A, Ai, n, A);  // D=A*C^T*A^{-1}

//        bitmat_print("D=A*C^T*A^{-1}=", A, n);

        ulong im = 0;  // == -i mod n
        for (ulong i=0; i<n; ++i)
        {
//            cout << " i=" << i << "  im=" << im << endl;
            ulong jm = im;  // == j-i mod n
            for (ulong j=0; j<n; ++j)
            {
                mSET(M, i, j, mGET(A, jm, im));
                ++jm;  if ( n==jm )  { jm = 0; }  // ++jm (mod n)
            }
            if ( 0==im )  { im=n-1; }  else  { --im; }  // --im (mod n)
        }

//        bitmat_print("M=", M, n);
    }

    return 1;
}
// -------------------------
