
#include "fxtio.h"

#include "bits/bithigh.h"
#include "bits/printbin.h"

#include "bits/bitsperlong.h"
#include "fxttypes.h"  // ulong


void
bitpol_print(ulong c)
// print as:  x^7+x^4+x^3+1
{
    ulong i = BITS_PER_LONG-1;
    ulong t = 1UL<<i;
    while ( c )
    {
        ulong q = c & t;
        if ( q )
        {
            c ^= q;
            if ( i<=1 )  cout << (i ? "x" : "1");
            else
            {
                cout << "x^" << i;
            }
            if ( c )  cout << "+";
        }
        t>>=1;
        --i;
    }
}
// -------------------------


void
bitpol_print_bin(ulong c)
// print as:  1..11.1
{
    ulong d = highest_bit_idx(c);
    print_bin_nn("", c, d+1);
}
// -------------------------


void
bitpol_print_coeff(ulong c)
// print as:  (7,4,3,0)
{
    cout << "(";
    ulong i = BITS_PER_LONG-1;
    ulong t = 1UL<<i;
    while ( c )
    {
        ulong q = c & t;
        if ( q )
        {
            c ^= q;
            cout << i;
            if ( c )  cout << ",";
        }
        t>>=1;
        --i;
    }
    cout << ")";
}
// -------------------------

void
bitpol_print_tex(ulong c)
// print as:  x^{7} + x^{4} + x^{3} + 1
{
//    cout << "$";
    ulong i = BITS_PER_LONG-1;
    ulong t = 1UL<<i;
    while ( c )
    {
        ulong q = c & t;
        if ( q )
        {
            c ^= q;
            if ( i<=1 )  cout << (i ? "x" : "1");
            else
            {
                cout << "x^" << "{" << i << "}";
            }
            if ( c )  cout << " + ";
        }
        t>>=1;
        --i;
    }
//    cout << "$";
}
// -------------------------


void
bitpol_print_factorization(const ulong *f, const ulong *e, ulong fct)
// print as:  (x+1)^5 *  (x^2+x+1)
{
    for (ulong i=0; i<fct; ++i)
    {
        cout << " (";
        bitpol_print(f[i]);
        cout << ")";
        if ( e[i]>1 )  cout << "^" << e[i];
        if ( i<fct-1 )  cout << " * ";
    }
}
// -------------------------

void
bitpol_print_bin_factorization(const ulong *f, const ulong *e, ulong fct)
// print as:  (1.)^5 (111)
{
    for (ulong i=0; i<fct; ++i)
    {
        cout << " (";
        bitpol_print_bin(f[i]);
        cout << ")";
        if ( e[i]>1 )  cout << "^" << e[i];
//        if ( i<fct-1 )  cout << " * ";
    }
}
// -------------------------


void
bitpol_print_coeff_factorization(const ulong *f, const ulong *e, ulong fct)
// print as:  (1,0)^5 * (2,1,0)
{
    for (ulong i=0; i<fct; ++i)
    {
        bitpol_print_coeff(f[i]);
        if ( e[i]>1 )  cout << "^" << e[i];
        if ( i<fct-1 )  cout << " * ";
    }
}
// -------------------------



void
bitpol_print_tex_factorization(const ulong *f, const ulong *e, ulong fct)
// print as:  \left(x + 1\right)^{5} \cdot  \left(x^{2} + x + 1\right)
{
    for (ulong i=0; i<fct; ++i)
    {
        cout << " \\left(";
        bitpol_print_tex(f[i]);
        cout << "\\right)";
        if ( e[i]>1 )  cout << "^{" << e[i] << "}";
        if ( i<fct-1 )  cout << " \\cdot ";
    }
}
// -------------------------
