
#include <cmath>

#include "fxttypes.h"
#include "complextype.h"

#include "aux0/sincos.h"
#include "aux0/csincos.h"
#include "aux1/copy.h"

void
make_fft_chirp(Complex *w, ulong n, ulong nn, int is)
// For k=0..n-1:   w[k] := exp( is * k*k * (i*2*PI/n)/2 )
//   where i = sqrt(-1)
// For k=n..nn-1:  w[k] = 0
{
    double phi = 1.0*is*M_PI/n;  // == (i*2*Pi/n)/2
    ulong k2 = 0,  n2 = 2*n;
    for (ulong k=0; k<n; ++k)
    {
        w[k] = SinCos(phi*k2);
        k2 += (2*k+1);
        if ( k2>n2 )  k2 -= n2;
        // here:  k2 == (k*k) mod 2*n;
    }
    null(w+n, nn-n);
}
// -------------------------

void
make_fft_chirp_n2(Complex *w, ulong n, ulong nn, int is)
// For k=0..2*n-1:   w[k] = exp( is * k*k * (i*2*PI/n)/2 )
//   where i = sqrt(-1)
// For k=2*n..nn-1:  w[k] = 0
{
    double phi = 1.0*is*M_PI/n;  // == (i*2*Pi/n)/2
    ulong n2 = n*2;
    ulong k2 = 0;
    for (ulong k=0; k<n2; ++k)
    {
        w[k] = SinCos(phi*k2);
        k2 += (2*k+1);
        if ( k2>n2 )  k2 -= n2;
        // here:  k2 == (k*k) mod 2*n;
    }
    null(w+n2, nn-n2);
}
// -------------------------


//void
//complete_fft_chirp(double *wr, double *wi, ulong n, ulong nn)
////
//// exp(sqrt(-1)*pi*k*k/n) for k>=n :
//// k==n+j, k*k==n*n+2*j*n+j*j
//// exp(sqrt(-1)*pi/n*k*k)==exp(+-sqrt(-1)*pi*j*j/n)
//// with plus for n even, minus for n odd
////
//{
//    if ( n&1 )  // n odd
//    {
//        for (ulong k=n; k<nn; ++k)  wr[k] = -wr[k-n];
//        for (ulong k=n; k<nn; ++k)  wi[k] = -wi[k-n];
//    }
//    else
//    {
//        for (ulong k=n; k<nn; ++k)  wr[k] = wr[k-n];
//        for (ulong k=n; k<nn; ++k)  wi[k] = wi[k-n];
//    }
//}
//// -------------------------


void
make_fft_fract_chirp(Complex *w, double v, ulong n, ulong nn)
// For k=0..nn:   w[k] == exp(sqrt(-1)*k*k*2*pi*/n/2)
{
    const double phi = v*2.0*M_PI/n/2;
    ulong n2 = 2*n;
    ulong np=0;
    for (ulong k=0; k<nn; ++k)
    {
        w[k] = SinCos(phi*np);

        np += ((k<<1)+1);  // np == (k*k)%n2
        if ( np>=n2 )  np -= n2;
    }
}
// -------------------------

void
make_fft_fract_chirp(double *wr, double *wi, double v, ulong n, ulong nn)
// For k=0..nn:   Complex(wr[k],wi[k]) == exp(sqrt(-1)*k*k*2*pi*/n/2)
{
    const double phi = v*2.0*M_PI/n/2;
    ulong n2 = 2*n;
    ulong np=0;
    for (ulong k=0; k<nn; ++k)
    {
        double c,s;
        SinCos(phi*np, &s, &c);
        wr[k] = c;
        wi[k] = s;

        np += ((k<<1)+1);  // np == (k*k)%n2
        if ( np>=n2 )  np -= n2;
    }
}
// -------------------------

