
#include "comb/combination-rec.h"
#include "fxttypes.h"

#include <cstdlib>  // rand(), srand()


//#define TAIL_RECURSION  // define to use tail recursion
#ifdef TAIL_RECURSION
#include "aux0/swap.h"
#endif


static inline bool rand_bit()
{
    uint r = rand();
    r ^= r>>16;
    r ^= r>>8;
    r ^= r>>4;
    return  r & 1;
}
// -------------------------

void
comb_rec::next_rec(ulong d)
{
    ulong r = k_ - d;  // number of elements remaining

//    if ( (rv1==n_-1-r) || (0==r) )  // gives repeated elements
    if ( 0==r )
    {
        ++ct_;  // count objects
        visit_(*this);
    }
    else
    {
        ulong rv1 = rv_[d-1];  // left neighbor
        ++rct_;  // measure computational work
        bool q;
        switch ( rq_ )
        {
        case 0:  q = 1;  break;   // lex order
        case 1:  q = !(d&1);  break;   // gray order
        case 2:  q = rv1&1;  break;  // near-perfect order
        case 3:  q = (d^rv1)&1;  break;  // diagonal gray order
        default: q = rand_bit();  // "random" order
//        default: q = 0;  // unreached, avoid compiler warning
        }

        if ( nq_ )  q = !q;

#ifndef TAIL_RECURSION
        if ( q )
        {
            for (ulong x=rv1+1; x<=n_-r; ++x)  // forward
            {
                rv_[d] = x;
                next_rec(d+1);
            }
        }
        else
        {
            for (ulong x=n_-r; (long)x>=(long)rv1+1; --x)  // backward
            {
                rv_[d] = x;
                next_rec(d+1);
            }
        }
#else //  TAIL_RECURSION
        long x0=rv1+1, x1=n_-r, i=1;  // forward
        if ( !q )  { swap2(x0, x1);  i=-i; }  // backward
        x1 += i;
        for (long x=x0; (long)x!=x1; x+=i)
        {
            rv_[d] = x;
            next_rec(d+1);
        }
#endif //  TAIL_RECURSION
    }
}
// -------------------------
