
#include "comb/setpartition-rgs.h"
#include "fxtiomanip.h"
#include "sort/minmaxmed23.h"  // max2()
#include "fxttypes.h"


#ifdef SETPART_RGS_ITERATIVE
// iterative function inlined
#else // SETPART_RGS_ITERATIVE
int
set_partition_rgs::next_rec(ulong k)
// Update partition in level k from partition in level k-1  (k<=n)
// Return number of sets in created partition
{
    if ( k<=1 )  return 0;  // current is last

    int d = d_[k];
    int as = as_[k] + d;
    bool ovq = ( (d>0) ? (as>ns_[k-1]) : (as<0) );
    if ( ovq )  // have to recurse
    {
        if ( 0==next_rec(k-1) )  return 0;

        d = ( xdr_ ? -d : dr0_ );
        d_[k] = d;

        as = ( (d>0) ? 0 : ns_[k-1] );
    }
    as_[k] = as;

    ulong ns = ns_[k] = max2(ns_[k-1], as_[k]+1);

    return ns;
}
// -------------------------
#endif // SETPART_RGS_ITERATIVE



void
set_partition_rgs::init(bool xdr, int dr0)
{
    dr0 = ( (dr0>=0) ? +1 : -1 );
    dr0_ = dr0;

    xdr_ = xdr;

    ulong n = n_;
    if ( dr0_>0 )
    {
        for (ulong k=0; k<=n; ++k)  as_[k] = 0;
        for (ulong k=0; k<=n; ++k)  ns_[k] = 1;
    }
    else
    {
        for (ulong k=0; k<=n; ++k)  as_[k] = k-1;
        for (ulong k=0; k<=n; ++k)  ns_[k] = k;
    }

    for (ulong k=0; k<=n; ++k)  d_[k] = dr0_;
}
// -------------------------


void
set_partition_rgs::print()  const
{
//    const char sep[]=", ";
    const char sep[]=" ";
    ulong n = n_;

    cout << "as[ ";
    for (ulong j=1; j<=n; ++j)  cout << as_[j] << sep;
    cout << "]    ";

    cout << "ns[ ";
    for (ulong j=1; j<=n; ++j)  cout << ns_[j] << sep;
    cout << "]    ";

    cout << "d[ ";
    for (ulong j=1; j<=n; ++j)  cout << (d_[j]>0 ? '+' : '-') << sep;
    cout << "]    ";
}
// -------------------------
