
#include "fxttypes.h"
#include "bits/bitsperlong.h"
#include "bits/bitasm.h"
#include "bits-all.h"
#include "bits/printbin.h"
#include "jjassert.h"
#include "fxtio.h"
#include "fxtiomanip.h"

#include <cstdlib> // rand()

char str[BYTES_PER_LONG];

#define PP(x) \
 cout << setw(25) << #x << " = "; \
 print_bin_nn("", (x),0); \
 cout << "  " << x; cout << endl;

static inline ulong hbi(ulong x)
// this is a _NON_WORKING_ highest_bit_idx() implementation
{
    ulong r = 0;
#if  BITS_PER_LONG >= 64
    if ( x & (~0UL<<32) )  r += 32;
#endif
    if ( x & 0xffff0000UL )  r += 16;
    if ( x & 0xff00ff00UL )  r +=  8;
    if ( x & 0xf0f0f0f0UL )  r +=  4;
    if ( x & 0xccccccccUL )  r +=  2;
    if ( x & 0xaaaaaaaaUL )  r +=  1;
    return r;
}
// -------------------------


//#define VERBOSE_TEST

void
do_some_tests(ulong v)
{
//#ifdef VERBOSE_TEST  // print to debug  ;-)
//    cout << "\n v = " << v  << hex << " = 0x" << v << dec << " = ";
//    cout << endl;
//    PP((v));
//    ulong h1 = highest_bit_idx(v);
//    ulong h2 = hbi(v);
//    PP((h1));
//    PP((h2));
//    jjassert( h1 == h2 );
//#endif // VERBOSE_TEST


    ulong t;

    jjassert( v == butterfly_2(butterfly_2(v)) );
    jjassert( v == bit_unzip(bit_zip(v)) );

    jjassert( (low_bits(v) & low_zeros(v)) == 0 );
    jjassert( (low_bits(v) | low_zeros(v)) != 0 );

    jjassert( high_bits(v) == high_zeros(~v) );
    jjassert( (high_bits(v) & high_zeros(v)) == 0 );
    jjassert( (high_bits(v) | high_zeros(v)) != 0 );

    jjassert( (highest_bit_01edge(v) & highest_bit_10edge(v)) == highest_bit(v) ); // CHECK

    t = highest_bit_01edge(v);  jjassert( bit_count(t) == bit_count_01(t) ); // CHECK

    jjassert( (lowest_bit_01edge(v) & lowest_bit_10edge(v)) == lowest_bit(v) ); // CHECK

    jjassert( (v==0) || (lowest_bit_idx(v) == (BITS_PER_LONG-1 - highest_bit_idx(revbin(v)))) ); // CHECK

    jjassert( (v!=0) || (lowest_bit_idx(v) == highest_bit_idx(revbin(v))) ); // CHECK

    jjassert( lowest_bit(v) == lowest_zero(~v) ); // CHECK
    jjassert( highest_bit(v) == highest_zero(~v) ); // CHECK
    jjassert( revbin( highest_bit(v) ) == lowest_bit( revbin(v) ) ); // CHECK
    jjassert( revbin(v) == revbin(v, BITS_PER_LONG) ); // CHECK

    jjassert( single_values(v) == single_bits(v) | single_zeros(v) ); // CHECK
    jjassert( single_bits(v)  == single_zeros(~v) ); // CHECK
    jjassert( single_bits(~v) == single_zeros(v) ); // CHECK

    jjassert( border_bits(v) == high_border_bits(v) | low_border_bits(v) ); // CHECK
//    jjassert( border_bits(v) == border_bits(~v) );  // not true

    jjassert( block_border_bits(v) == low_block_border_bits(v) ^ high_block_border_bits(v) ); // CHECK
    jjassert( block_bits(v) == block_border_bits(v) | interior_bits(v) ); // CHECK

    jjassert( v == block_bits(v) | single_bits(v) ); // CHECK
    jjassert( bit_count(v) == bit_count_sparse(v) ); // CHECK

    jjassert( interior_bits(v) == (interior_bits(v) & interior_values(v)) ); // CHECK

//    jjassert( (interior_bits(v) | interior_bits(~v)) == interior_values(v) ); //  not true

    jjassert( parity(v) == (inverse_gray_code(v) & 1) ); // CHECK

    ulong gxg = inverse_green_code(v) ^ inverse_gray_code(v);
    if ( parity(v) )  jjassert( (~v) == gxg ); // CHECK
    else    jjassert( (v) == gxg ); // CHECK

//    jjassert( (~inverse_green_code(v)) == (inverse_gray_code(v)>>1) ); // not true

    if ( v ) { jjassert( (1UL<<ld(v)) <= v ); }  else { jjassert( ld(v) == 0 ); } // CHECK

    for (ulong r=0; r<(BITS_PER_LONG); ++r)
    {
        jjassert( v==bit_rotate_right(bit_rotate_left(v,r), r) ); // CHECK
        jjassert( v==bit_rotate_right(bit_rotate_right(v,r), BITS_PER_LONG-r) ); // CHECK
        jjassert( v==bit_rotate_left(bit_rotate_left(v,r), BITS_PER_LONG-r) ); // CHECK
    }

    if ( contains_zero_byte(v) )
    {
        ulong x = v;
        for (ulong k=0; k<BYTES_PER_LONG; ++k)
        {
            str[k] = (char)(x & 0xff);
            x >>= 8;
        }
        ulong k;
        for (k=0; k<BYTES_PER_LONG; ++k)  if ( 0==str[k] )  break;
        if ( k==BYTES_PER_LONG )
        {
            cout << hex << " contains_zero_byte( 0x" << v << " )";
            cout        << "   gave  0x" << contains_zero_byte(v) << endl;
            jjassert2(0, " contains_zero_byte() failed ");
        }
    }
}
// -------------------------


int
main()
{
    cout << "Running tests for low level (bit-) functions "
#ifdef  NO_BITS_USE_ASM
        "WITHOUT asm-functions"
#else
        "with ASM-versions"
#endif
         "..." << endl;
//    cout << "RAND_MAX=" << (RAND_MAX) << endl;

    // some special values we'd like to test:

    for (ulong x=0; x<100; x++)  do_some_tests( x );

    for (ulong x=1; x; x<<=1)
    {
        do_some_tests( x );    // one bit
        do_some_tests( x-1 );  // low burst
        do_some_tests( ~(x-1) );  // high burst
    }

    do_some_tests( 0 );
    do_some_tests( ~0UL );

    ulong x = 1UL<<(BITS_PER_LONG-1);  // highest bit set
    do_some_tests( x );
    do_some_tests( x + 1 );
    do_some_tests( x - 1 );


    ulong n = 300*1000;  // this many tests with random values
    for (ulong i=0; i<n; ++i)
    {
        ulong v = (ulong)rand();
        v ^= ((ulong)rand())<<16; // also want high bit !=0

        do_some_tests(v);
        if ( 0==(i%1024) )  { cout << "."; cout.flush(); }
    }

    cout << "\nOK passed " << n << " tests." << endl;

    return 0;
}
// -------------------------
